/*******************************************************************************

     Copyright (c) 2014-2015, Analog Devices, Inc.  All rights reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted (subject to the limitations in the
     disclaimer below) provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the
       distribution.

     * Neither the name of Analog Devices, Inc.  nor the names of its
       contributors may be used to endorse or promote products derived
       from this software without specific prior written permission.

     NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE
     GRANTED BY THIS LICENSE.  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
     HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
     WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
     LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
     CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
     SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
     BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
     OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
     IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

/*******************************************************************************
    File name:    main.c
    Purpose:      Provide main entry point to invoke initcode from standard CRT
                  application.
    Notes:        initcode doesn't use the ADI_ROM_BOOT_CONFIG pointer, so NULL
                  is passed.
*******************************************************************************/

#include <cdefSC57x_rom.h>
#include <stddef.h>
#include <runtime/cache/adi_cache.h>
#include <stdint.h>

void initcode(ADI_ROM_BOOT_CONFIG* pBootStruct /*unused*/);
void disable_mmu(void);

/* Set the global control variable adi_cache_gEnable to 0 so the cache runtime
 * support does not enable either of the L1 or L2 caches. This is done so that
 * emulator doesn't attempt to invalidate caches while loading the application
 * as doing so increases load times considerably.
 */
uint32_t adi_cache_gEnable = 0uL;

/* Define a stub adi_cache_Init function to replace the default one
 * to ensure caches aren't enabled and to avoid cache support functions
 * being linked in unnecessarily.
 */
ADI_CACHE_RESULT adi_cache_Init(void)
{
   return ADI_CACHE_SUCCESS;
}

int main(void)
{
  initcode(NULL);

  /* Disable the MMU and cache before returning control to the emulator */
  disable_mmu();

  return 0;
}
