/*
 * Copyright (c) 2014, Analog Devices, Inc.  All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted (subject to the limitations in the
 * disclaimer below) provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 * * Neither the name of Analog Devices, Inc.  nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE
 * GRANTED BY THIS LICENSE.  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT
 * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* This is an example MMU configuration function, used to resolve the
 * adi_mmu_Init symbol required by the adsp-sc58x's CRTs.
 */

 /* First Level Descriptor Macros */
 #define FLD_TYPE_SECTION             0x00002
 #define FLD_MEM_DEVICE_SHAREABLE     0x00004
 #define FLD_MEM_DEVICE_NON_SHAREABLE 0x02000
 #define FLD_MEM_NORMAL_NONCACHEABLE  0x01000
 #define FLD_MEM_SHAREABLE            0x10000 /* Only valid in Normal memory */
 #define FLD_DOM_15                   0x001E0
 #define FLD_AP_FULL_ACCESS           0x00C00

    .syntax unified
#if defined(__ADSPCORTEXA5__)
    .cpu cortex-a5
    .eabi_attribute 27, 3    @ Tag_ABI_HardFP_use
    .eabi_attribute 28, 1    @ Tag_ABI_VFP_args
    .fpu neon-vfpv4
#elif defined(__ADSPCORTEXM3__)
    .cpu cortex-m3
    .fpu softvfp
#else
#error Unsupported part.
#endif
    .eabi_attribute 20, 1    @ Tag_ABI_FP_denormal
    .eabi_attribute 21, 1    @ Tag_ABI_FP_exceptions
    .eabi_attribute 23, 3    @ Tag_ABI_FP_number_model
    .eabi_attribute 24, 1    @ Tag_ABI_align8_needed
    .eabi_attribute 25, 1    @ Tag_ABI_align8_preserved
    .eabi_attribute 26, 1    @ Tag_ABI_enum_size
    .eabi_attribute 30, 2    @ Tag_ABI_optimization_goals
    .eabi_attribute 34, 1    @ Tag_CPU_unaligned_access
    .eabi_attribute 18, 4    @ Tag_ABI_PCS_wchar_t
    .thumb
    .text
    .align    2
    .global    adi_mmu_Init
    .thumb
    .thumb_func
    .type    adi_mmu_Init, %function
adi_mmu_Init:
    @ args = 0, pretend = 0, frame = 0
    @ frame_needed = 0, uses_anonymous_args = 0
    @ link register save eliminated.
#if defined(__ADSPCORTEXA5__)
#define SETUP_L2_TTB 1
#define SETUP_DDR 1
#if defined(SETUP_L2_TTB)
/* Set up translation tables and enable MMU */

/*  0x0000 0000 - 1FFF FFFF - Boot code and config regs */
    /* Build page descriptor */
    movw r0, #(FLD_TYPE_SECTION | FLD_MEM_DEVICE_NON_SHAREABLE | \
               FLD_DOM_15 | FLD_AP_FULL_ACCESS)
    ldr r1, =__ttbaddr
    mov r3, #0x0                 /* Loop counter */
.Lwrite_loopA:
    orr r2, r0, r3, LSL #20      /* OR addr & default PTE bits */
    str r2, [r1, r3, LSL #2]     /* write page entry to translation table */
    add r3, r3, #1               /* Increment loop counter */
    cmp r3, #0x200               /* End of range */
    bne .Lwrite_loopA

/*  0x2000 0000 - 2FFF FFFF - L2 Memory */
    /* Build page descriptor */
    movw r0, #(FLD_TYPE_SECTION | FLD_MEM_NORMAL_NONCACHEABLE | \
               FLD_DOM_15 | FLD_AP_FULL_ACCESS)
    orr r0, #FLD_MEM_SHAREABLE
.Lwrite_loopB:
    orr r2, r0, r3, LSL #20      /* OR addr & default PTE bits */
    str r2, [r1, r3, LSL #2]     /* write page entry to translation table */
    add r3, r3, #1               /* Increment loop counter */
    cmp r3, #0x300               /* End of range */
    bne .Lwrite_loopB

/*  0x3000 0000 - 7FFF FFFF - MMRs, Async, PCIe and SPI */
    /* Build page descriptor */
    movw r0, #(FLD_TYPE_SECTION | FLD_MEM_DEVICE_SHAREABLE | \
               FLD_DOM_15 | FLD_AP_FULL_ACCESS)
.Lwrite_loopC:
    orr r2, r0, r3, LSL #20      /* OR addr & default PTE bits */
    str r2, [r1, r3, LSL #2]     /* write page entry to translation table */
    add r3, r3, #1               /* Increment loop counter */
    cmp r3, #0x800               /* End of range */
    bne .Lwrite_loopC

#if defined(SETUP_DDR)
/*  0x8000 0000 - FFFF FFFF - DDR */
    /* Build page descriptor */
    movw r0, #(FLD_TYPE_SECTION | FLD_MEM_NORMAL_NONCACHEABLE | \
               FLD_DOM_15 | FLD_AP_FULL_ACCESS)
    orr r0, #FLD_MEM_SHAREABLE
.Lwrite_loopD:
    orr r2, r0, r3, LSL #20      /* OR addr & default PTE bits */
    str r2, [r1, r3, LSL #2]     /* write page entry to translation table */
    add r3, r3, #1               /* Increment loop counter */
    cmp r3, #0x1000              /* End of range */
    bne .Lwrite_loopD
#endif /* SETUP_DDR */
    mcr p15, 0, r0, c8, c7, 0    /* Invalidate TLB */
    isb
    /* Initialize MMU */
    ldr r1, =__ttbaddr
    mcr p15, 0, r1, c2, c0, 0    /* Write Translation Table Base Register 0 */
    mov r2, #0x0
    mcr p15, 0, r2, c2, c0, 2    /* Write Translation Table Base Control Reg */
    /* Set all domains to Manager */
    mvn r1, #0x0
    mcr p15, 0, r1, c3, c0, 0    /* Write Domain Access Control Register */
#endif /* SETUP_TTB */
    /* Enable MMU */
    mrc p15, 0, r1, c1, c0, 0    /* Read System Control Register */
   	bic r1, r1, #(0x1 << 12)   /* Disable I-Cache */
	bic r1, r1, #(0x1 <<  2)   /* Disable D-Cache */
    orr r1, r1, #0x1             /* Bit 0 is MMU enable */
    mcr p15, 0, r1, c1, c0, 0    /* Write System Control Register */
    dsb
    isb
#endif /* defined(__ADSPCORTEXA5__) */
    bx lr
    .size adi_mmu_Init, .-adi_mmu_Init
